#ifndef _SF_XML_PARSER_H
#define _SF_XML_PARSER_H

#include <windows.h>
#include <vector>
#include "CommonUtility.H"
#include "ResourceManager.H"

using namespace std;

#define	MAX_XML_TOKEN_NAME_LENGTH	128
struct	stTokenNode
{
	char	m_strTokenName[MAX_XML_TOKEN_NAME_LENGTH+1];
	int	m_iLineNumber;
	int	m_iColumnNumber;

public:

	stTokenNode(char *strTokenName,int iLineNumber,int iColumnNumber)
	{
		SetToken(strTokenName,iLineNumber,iColumnNumber);
	}
	void	SetToken(char *strTokenName,int iLineNumber,int iColumnNumber)
	{
		strcpy(m_strTokenName,strTokenName);
		m_iLineNumber = iLineNumber;
		m_iColumnNumber = iColumnNumber;
	}
    char* GetTokenName() { return m_strTokenName; }
    int GetTokenNameByInt() { return atoi(m_strTokenName); }
    float GetTokenNameByFloat() { return atof(m_strTokenName); }
    int GetLineNumber() { return m_iLineNumber; }
    int GetColumnNumber() { return m_iColumnNumber; }

    bool equals(char *str)
    {
        return strcmp(m_strTokenName,str)==0;
    }
};

typedef vector<stTokenNode> TokenNodeVector;
typedef TokenNodeVector::iterator TokenNodeVectorItor;

typedef vector<char> CharVector;
typedef CharVector::iterator CharVectorItor;


class	SFTokenBuilder
{

public:

	enum	ERROR_CODE
	{
		EC_NOERROR=0,
		EC_FILEREADERROR,
		EC_CREATESTRINGBUFFERFAILED,
		EC_BUILDTOKENFAILED
	};

private:

	ERROR_CODE	m_LastErrorCode;
	TokenNodeVector	m_vTokenNode;
	CharVector	m_vSeperator;
	CharVector	m_vIgnoreChar;
	bool m_bUseQutationMark;
	int m_iCurrentParsePos;
	int m_iCurrentParseLine ,m_iCurrentParseColumn ;
	bool m_bNowFindEndQutation;
	int m_iFindEndQutationStartLine,m_iFindEndQutationStartColumn;

public:

	SFTokenBuilder()	{	Create();	}
	~SFTokenBuilder()	{	Destroy();	}

	int GetTotalTokenNumber()	{	return	m_vTokenNode.size();	}
	stTokenNode*	GetToken(int iNumber)
	{
		if(iNumber<0 || iNumber>=m_vTokenNode.size())
		{
            ODS("SFTokenBuilder::GetToken() Error, iNumber[%d] is out of range",iNumber);
			return NULL;
		}

		return &(m_vTokenNode[iNumber]);
	}
	bool BuildTokenFromFile(char* Filename);
	bool BuildTokenFromBuffer(CFileResource &FileRes);
	bool GetNextToken(CFileResource &FileRes,stTokenNode &Token);
	bool AddIgnoreChar(char IgnoreChar)
    {
        m_vIgnoreChar.push_back(IgnoreChar);
        return true;
    }
	bool IsIgnoreChar(char *ch)
	{
		if(strlen(ch)>1) return false;
		for(CharVectorItor Itor = m_vIgnoreChar.begin();Itor != m_vIgnoreChar.end(); Itor ++)
		{
			if(*Itor == *ch) return true;
		}
		return false;
	}
	bool	AddSeperator(char ch)
	{
		m_vSeperator.push_back(ch);
		return true;
	}
	bool IsSeperator(char *ch)
	{
		if(strlen(ch)>1) return false;
		for(CharVectorItor Itor = m_vSeperator.begin();Itor != m_vSeperator.end(); Itor ++)
		{
			if(*Itor == *ch) return true;
		}
		return false;
	}
	bool	IsQutationMark(char *ch)
	{
		if(strlen(ch)>1) return false;
		if(*ch == '\"') return true;
		return	false;
	}

	void	SetUseQutationMark(bool bUse)
	{
		m_bUseQutationMark = bUse;
	}
	bool	GetUseQutationMark()	{	return	m_bUseQutationMark;	}
	void	SetLastErrorCode(ERROR_CODE ErrorCode)
	{
		m_LastErrorCode = ErrorCode;
	}
	ERROR_CODE GetLastErrorCode()
    {
        return m_LastErrorCode;
    }
	void PrintContentToDebugTab();

private:

	void	Create();
	void	Destroy();

};


class SFXMLContentAttributeNode
{
    char m_AttrName[MAX_XML_TOKEN_NAME_LENGTH+1];
    char m_AttrValue[MAX_XML_TOKEN_NAME_LENGTH+1];

public:

    SFXMLContentAttributeNode()
    {
        SetNameAndValue("","");
    }
    SFXMLContentAttributeNode(char *Name,char *Value)
    {
        SetNameAndValue(Name,Value);
    }
    void SetNameAndValue(char* Name,char* Value)
    {
		assert(Name);
		assert(Value);
		assert(strlen(Name)<=MAX_XML_TOKEN_NAME_LENGTH);
		assert(strlen(Value)<=MAX_XML_TOKEN_NAME_LENGTH);
        
		strcpy(m_AttrName,Name);
        strcpy(m_AttrValue,Value);
    }
    char* GetName() { return m_AttrName; }
    char* GetValue() { return m_AttrValue; }
	int	GetValueByInt()	{	return	atoi(m_AttrValue);	}
	float GetValueByFloat()	{	return	atof(m_AttrValue);	}
};

typedef vector<SFXMLContentAttributeNode>	SFXMLContentAttributeNodeVector;
typedef TokenNodeVector::iterator SFXMLContentAttributeNodeItor;

class SFXMLContentAttribute
{
    SFXMLContentAttributeNodeVector m_vAttribute; 

public:

    SFXMLContentAttribute()
    {
    }
    bool AddAttributeNode(char* AttrName,char* AttrValue)
    {
		SFXMLContentAttributeNode NewNode(AttrName,AttrValue);
        m_vAttribute.push_back(NewNode);
        return true;
    }
    int GetTotalAttributeNodeNumber()
    {
        return m_vAttribute.size();
    }
	SFXMLContentAttributeNode*	GetAttributeNodeByName(char *strAttrName)
	{
		for(int i=0;i<m_vAttribute.size();i++)
		{
			if(strcmp(m_vAttribute[i].GetName(),strAttrName)==0) return &m_vAttribute[i];
		}
		return	NULL;
	}
    SFXMLContentAttributeNode* GetAttributeNode(int iNumber)
    {
        if(iNumber<0 || iNumber>=m_vAttribute.size())
        {
            ODS("SFXMLContentAttribute::GetAttributeNode() Error, iNumber[ %d ] is not correct",iNumber);
            return NULL;
        }

        return &(m_vAttribute[iNumber]);
    }
};
class	SFXMLContentData;
typedef vector<SFXMLContentData*>	SFXMLContentDataVector;
typedef SFXMLContentDataVector::iterator SFXMLContentDataItor;

class SFXMLContentData
{
    char m_DataName[MAX_XML_TOKEN_NAME_LENGTH+1];
    char m_DataValue[MAX_XML_TOKEN_NAME_LENGTH+1];
    SFXMLContentAttribute m_Attribute;
    SFXMLContentDataVector m_ChildContentData; 
    SFXMLContentData *m_pParentContentData;

public:

    SFXMLContentData()
    {
        m_DataName[0]='\0';
        m_DataValue[0]='\0';
        m_pParentContentData = NULL;
    }
	~SFXMLContentData()
	{
		for(SFXMLContentDataItor Itor = m_ChildContentData.begin();Itor != m_ChildContentData.end(); Itor ++)
		{
			delete (*Itor);
		}
	}

    SFXMLContentAttribute* GetAttribute() { return &m_Attribute; }
	char*	GetAttrValue(char *strAttrName)
	{
		SFXMLContentAttributeNode	*pNode = GetAttribute()->GetAttributeNodeByName(strAttrName);
		if(!pNode) return "";

		return pNode->GetValue();
	}
	int	GetAttrValueByInt(char *strAttrName)
	{
		SFXMLContentAttributeNode	*pNode = GetAttribute()->GetAttributeNodeByName(strAttrName);
		if(!pNode) return 0;

		return pNode->GetValueByInt();
	}
	float	GetAttrValueByFloat(char *strAttrName)
	{
		SFXMLContentAttributeNode	*pNode = GetAttribute()->GetAttributeNodeByName(strAttrName);
		if(!pNode) return 0.0f;

		return pNode->GetValueByFloat();
	}

    SFXMLContentData* AddChildContent()
    {
        SFXMLContentData *pChildContentData = new SFXMLContentData;
		pChildContentData->SetParentContentData(this);
        m_ChildContentData.push_back(pChildContentData);
        return pChildContentData;
    }
	void	SetParentContentData(SFXMLContentData *pParent)
	{
		m_pParentContentData = pParent;
	}
    SFXMLContentData* GetParentContentData()
    {
        return m_pParentContentData;
    }
    int GetTotalChildNumber()
    {
        return m_ChildContentData.size();
    }
    SFXMLContentData* GetChildData(int iIndex)
    {
        if(iIndex<0 || iIndex>=m_ChildContentData.size())
        {
            ODS("SFXMLContentData::GetChildData() Error, iIndex %d is out of range",iIndex);
            return NULL;
        }
        return m_ChildContentData[iIndex];
    }
    void SetDataName(char* Name) 
	{ 
		assert(Name);
		assert(strlen(Name)<=MAX_XML_TOKEN_NAME_LENGTH);
		strcpy(m_DataName,Name); 
	}
    void SetDataValue(char* Value)  
	{ 
		assert(Value);
		assert(strlen(Value)<=MAX_XML_TOKEN_NAME_LENGTH);
		strcpy(m_DataValue,Value); 
	}
    char* GetDataName() { return m_DataName; }
    char* GetDataValue() { return m_DataValue; }

};

class	SFXMLContent
{
public:

	enum	ERROR_CODE
	{
		EC_NOERROR = 0
	};
	
	ERROR_CODE	m_LastErrorCode;
	float m_fXMLVersion;
	char	m_EncodingCharSet[MAX_XML_TOKEN_NAME_LENGTH+1];
	SFXMLContentData	m_XMLContentData;

public:

    SFXMLContent() {
        m_fXMLVersion = -1;
        strcpy(m_EncodingCharSet,"UTF-8");
    }
    SFXMLContentData* GetContentData()
    {
        return &m_XMLContentData;
    }
    float GetXMLVersion() { return m_fXMLVersion; }
    char* GetEncodingCharSet() { return m_EncodingCharSet; }

    void SetXMLVersion(float fVersion) { m_fXMLVersion = fVersion; }
    void SetEncodingCharSet(char *CharSet) { strcpy(m_EncodingCharSet,CharSet); }

};
class	SFXMLParser
{
	enum	ERROR_CODE
	{
		EC_NOERROR=0,
		EC_BUILDTOKENFAILED,
		EC_BUILDXMLCONTENTFAILED,
		EC_NOTXMLFILE,
		EC_DECODINGERROR,
		EC_DECODEXMLHEADERFAILED,
		EC_DECODECONTENTDATAFAILED,
		EC_NOTCORRECTNAME,
		EC_GETNEXTTOKENFAILED
	};

	ERROR_CODE m_LastErrorCode;

	int	m_iNextTokenNumber;
	SFTokenBuilder	*m_pTokenBuilder;

public:

	SFXMLParser()	
	{
		m_iNextTokenNumber = 0;
		m_LastErrorCode = EC_NOERROR;
		m_pTokenBuilder = NULL;
	}
	~SFXMLParser()
	{
		if(m_pTokenBuilder) delete m_pTokenBuilder;	m_pTokenBuilder=NULL;
	}
    SFXMLContent* ParseXMLFile(char* FileName);
	bool BuildXMLContentStart(SFXMLContent *pDestXMLContent);
	bool DecodeContentData(SFXMLContentData *pDestContentData); // recursive method
    stTokenNode* GetNextToken()
    {
        if(m_iNextTokenNumber>=m_pTokenBuilder->GetTotalTokenNumber())
        {
            SetLastErrorCode(EC_GETNEXTTOKENFAILED);
            return NULL;
        }

        return m_pTokenBuilder->GetToken(m_iNextTokenNumber++);
    }
	int DecodeContentDataHeader(SFXMLContentData *pDestContentData);
	bool IsCorrectForContentDataName(stTokenNode *pToken);
	bool DecodeXMLHeader(SFXMLContent *pDestXMLContent);

    void SetLastErrorCode(ERROR_CODE ErrorCode,stTokenNode *pErrorToken)
    {
        ODS("SFXMLParserError : Code %d Token : %s L: %d C: %d\n",ErrorCode,pErrorToken->GetTokenName(),pErrorToken->GetLineNumber(),pErrorToken->GetColumnNumber());
        m_LastErrorCode = ErrorCode;
    }
    void SetLastErrorCode(ERROR_CODE ErrorCode)
    {
        ODS("SFXMLParserError : Code %d\n",ErrorCode);
        m_LastErrorCode = ErrorCode;
    }
    ERROR_CODE GetLastErrorCode()
    {
        return m_LastErrorCode;
    }
	void PrintContentToDebugTab(SFXMLContent *pDestXMLContent);
	void PrintContentToDebugTab(SFXMLContentData *pContentData,int iTabCount);
};


#endif