#include "Direct3D.H"
#include "D3DX9Tex.H"
#include "DXErr9.H"
#include "assert.H"
#include <stdio.h>
#include <ddraw.h>

CD3D90* CD3D90::ms_Singleton;
bool	CD3D90::Create()
{
	m_pD3D = NULL;
	m_pd3dDevice = NULL;
	m_pD3DSprite = NULL;
	memset(&m_PresentParam,0,sizeof(D3DPRESENT_PARAMETERS));
	memset(&m_D3DDM,0,sizeof(D3DDISPLAYMODE));
	memset(&m_Caps,0,sizeof(D3DCAPS9));
	m_bIsSoftwareVertexProcessing = false;
	memset(&m_matProjection,0,sizeof(D3DXMATRIX));
	memset(&m_matWorld,0,sizeof(D3DXMATRIX));
	memset(&m_matView,0,sizeof(D3DXMATRIX));

	m_dwTotalVideoMemory = 0;
	m_dwTotalVertexBufferMemorySize[0]=m_dwTotalVertexBufferMemorySize[1]=m_dwTotalVertexBufferMemorySize[2]=0;
	m_dwTotalIndexBufferMemorySize[0]=m_dwTotalIndexBufferMemorySize[1]=m_dwTotalIndexBufferMemorySize[2]=0;
	m_dwTotalTextureMemorySize[0]=m_dwTotalTextureMemorySize[1]=m_dwTotalTextureMemorySize[2]=0;
	m_dwTotalMemorySize[0]=m_dwTotalMemorySize[1]=m_dwTotalMemorySize[2];

	return	true;
}
bool	CD3D90::Destroy()
{
	if(m_pD3DSprite) m_pD3DSprite->Release();
	m_pD3DSprite=NULL;

	if(m_pd3dDevice) m_pd3dDevice->Release();
	m_pd3dDevice=NULL;

	if(m_pD3D) m_pD3D->Release();
	m_pD3D=NULL;

	return	true;
}

bool	CD3D90::Init(HWND hWnd,bool bFullScreen,int iScreenWidth,int iScreenHeight,int iRefreshRate)
{
	HRESULT hr;

	    // Create the D3D object.
    if( NULL == ( m_pD3D = Direct3DCreate9( D3D_SDK_VERSION ) ) )
        return false;


    if( FAILED( m_pD3D->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &m_D3DDM ) ) )
        return E_FAIL;

    // Set up the structure used to create the D3DDevice
    ZeroMemory( &m_PresentParam, sizeof(m_PresentParam) );

	if(bFullScreen)
	{
		m_PresentParam.Windowed               = false;
		m_PresentParam.BackBufferCount        = 1;
		m_PresentParam.SwapEffect             = D3DSWAPEFFECT_FLIP;
		m_PresentParam.EnableAutoDepthStencil = TRUE;
		m_PresentParam.hDeviceWindow          = hWnd;
		m_PresentParam.MultiSampleType        = D3DMULTISAMPLE_NONE ;
		m_PresentParam.MultiSampleQuality     = 0;

		m_PresentParam.BackBufferWidth  = iScreenWidth;
		m_PresentParam.BackBufferHeight = iScreenHeight;
		m_PresentParam.BackBufferFormat = m_D3DDM.Format;
		m_PresentParam.FullScreen_RefreshRateInHz = iRefreshRate;
		m_PresentParam.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;
		m_PresentParam.AutoDepthStencilFormat = D3DFMT_D16;
	}
	else
	{
		m_PresentParam.Windowed = TRUE;
		m_PresentParam.SwapEffect = D3DSWAPEFFECT_FLIP;
		m_PresentParam.BackBufferFormat = m_D3DDM.Format;
		m_PresentParam.EnableAutoDepthStencil = TRUE;
		m_PresentParam.AutoDepthStencilFormat = D3DFMT_D16;
        m_PresentParam.Flags              = 0;
		m_PresentParam.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;
	}

	m_bIsSoftwareVertexProcessing=false;

    // Create the D3DDevice
    if( FAILED( hr=m_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
                                      D3DCREATE_HARDWARE_VERTEXPROCESSING | D3DCREATE_MULTITHREADED,
                                      &m_PresentParam, &m_pd3dDevice ) ) )
    {
		m_pD3D->Release();
		m_pD3D=NULL;
		MessageBox(NULL,DXGetErrorString9(hr),"IDirect3D9::CreateDevice() Error",MB_OK);
        return false;
    }

	m_pd3dDevice->GetDeviceCaps(&m_Caps);

	if(FAILED( hr = D3DXCreateSprite(m_pd3dDevice,&m_pD3DSprite) ) )
	{
		m_pD3D->Release();
		m_pD3D=NULL;

		MessageBox(NULL,DXGetErrorString9(hr),"D3DXCreateSprite() Error",MB_OK);
        return false;
	}


	//! Ʈ ʱⰪ 
	D3DXMatrixIdentity(&m_matWorld);
	D3DXVECTOR3 vEyePt( 0.0f, 0.0f,-10.0f );
	D3DXVECTOR3 vLookatPt( 0.0f, 0.0f, 0.0f );
	D3DXVECTOR3 vUpVec( 0.0f, 1.0f, 0.0f );
	D3DXMatrixLookAtLH( &m_matView, &vEyePt, &vLookatPt, &vUpVec );
	D3DXMatrixPerspectiveFovLH( &m_matProjection, D3DX_PI/4, GetScreenSizeAspect(), 1.0f, 10000.0f );

	InitMatrices();
	SetupRenderEnv();


   LPDIRECTDRAW7 lpDD = NULL;
   DDSCAPS2      ddsCaps2; 
   DWORD         dwTotal; 
   DWORD         dwFree;

   hr = DirectDrawCreateEx(NULL, (VOID**)&lpDD, IID_IDirectDraw7, NULL );
   if (FAILED(hr))
	   return FALSE; 

   // Initialize the structure.
   ZeroMemory(&ddsCaps2, sizeof(ddsCaps2));

   ddsCaps2.dwCaps = DDSCAPS_VIDEOMEMORY | DDSCAPS_LOCALVIDMEM; 
   hr = lpDD->GetAvailableVidMem(&ddsCaps2, &dwTotal, &m_dwTotalVideoMemory); 
   if (FAILED(hr))
	   return FALSE;

   if(lpDD)
	   lpDD->Release();

	return	true;	
}
bool	CD3D90::BeginScene(DWORD dwClearColor)
{
	if(!m_pd3dDevice) return false;
	if(SUCCEEDED( m_pd3dDevice->BeginScene() ))
	{
		HRESULT hr=m_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,
							 dwClearColor, 1.0f, 0L );
		if(FAILED(hr))
		{
			MessageBox(NULL,DXGetErrorString9(hr),"Clear Error",MB_OK);
		}
	}
	else return false;

	return	true;
}
bool	CD3D90::EndScene()
{
	if(!m_pd3dDevice) return false;
	return	SUCCEEDED( m_pd3dDevice->EndScene() );
}
bool	CD3D90::Present()
{
	if(!m_pd3dDevice) return false;
	return	SUCCEEDED( m_pd3dDevice->Present( NULL, NULL, NULL, NULL ) );
}
bool	CD3D90::SetTexture(LPDIRECT3DTEXTURE9 pTexture,int iStage)
{
	if(!m_pd3dDevice) return false;
	m_pd3dDevice->SetTexture(iStage,pTexture);
	return	true;
}
void	CD3D90::InitMatrices()
{
	m_pd3dDevice->SetTransform( D3DTS_WORLD, &m_matWorld );
	m_pd3dDevice->SetTransform( D3DTS_VIEW, &m_matView );
	m_pd3dDevice->SetTransform( D3DTS_PROJECTION, &m_matProjection );
}

D3DXMATRIX*	CD3D90::SetViewMatrix(float fEyeX,float fEyeY,float fEyeZ,float fLookAtX,float fLookAtY,float fLookAtZ,float fUpX,float fUpY,float fUpZ)
{
	D3DXVECTOR3 vEyePt( fEyeX, fEyeY,fEyeZ );
	D3DXVECTOR3 vLookatPt(fLookAtX ,fLookAtY, fLookAtZ);
	D3DXVECTOR3 vUpVec( fUpX,fUpY,fUpZ);
	D3DXMatrixLookAtLH( &m_matView, &vEyePt, &vLookatPt, &vUpVec );
	if(m_pd3dDevice)
		m_pd3dDevice->SetTransform( D3DTS_VIEW, &m_matView );

	return	&m_matView;
}
void	CD3D90::SetViewMatrix(D3DXMATRIX *pMatrix)
{
	m_matView=*pMatrix;

	if(m_pd3dDevice)
		m_pd3dDevice->SetTransform( D3DTS_VIEW, &m_matView );
}
void CD3D90::SetWorldMatrix(D3DXMATRIX *pMatrix)
{
	m_matWorld=*pMatrix;
	if(m_pd3dDevice) m_pd3dDevice->SetTransform( D3DTS_WORLD, &m_matWorld );
}
void	CD3D90::SetOrthoProjectionMatrix(float fW,float fH,float fZn,float fZf)
{
	D3DXMatrixOrthoLH(&m_matProjection,fW,fH,fZn,fZf);
	if(m_pd3dDevice) m_pd3dDevice->SetTransform( D3DTS_PROJECTION, &m_matProjection );
}
void	CD3D90::SetPerspectiveProjectionMatrix(float fFovY,float fAspect,float fZn,float fZf)
{
	D3DXMatrixPerspectiveFovLH( &m_matProjection, fFovY, fAspect, fZn, fZf );
	if(m_pd3dDevice) m_pd3dDevice->SetTransform( D3DTS_PROJECTION, &m_matProjection );
}
void	CD3D90::SetProjectionMatrix(D3DXMATRIX *pMatrix)
{
	m_matProjection=*pMatrix;
	if(m_pd3dDevice) m_pd3dDevice->SetTransform( D3DTS_PROJECTION, &m_matProjection );
}
float	CD3D90::GetScreenSizeAspect()
{
	return	((FLOAT)m_PresentParam.BackBufferWidth) / ((float)m_PresentParam.BackBufferHeight);
}

void	CD3D90::SetupRenderEnv()
{
    m_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE );
    m_pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE );
    m_pd3dDevice->SetRenderState( D3DRS_ZENABLE, TRUE );
	m_pd3dDevice->SetRenderState(D3DRS_SRCBLEND,D3DBLEND_SRCALPHA);
	m_pd3dDevice->SetRenderState(D3DRS_DESTBLEND,D3DBLEND_INVSRCALPHA);
	m_pd3dDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
	m_pd3dDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL);
	m_pd3dDevice->SetRenderState(D3DRS_ALPHAREF, 0x80);
}

bool	CD3D90::SetBackBufferSize(int iWidth,int iHeight)
{
	if(!m_pd3dDevice) return false;
	if(iWidth==m_PresentParam.BackBufferWidth && iHeight==m_PresentParam.BackBufferHeight)
		return	true;
	if(iWidth<0) iWidth=0;
	if(iHeight<0) iHeight=0;

	m_PresentParam.BackBufferWidth=iWidth;
	m_PresentParam.BackBufferHeight=iHeight;

	if(m_pd3dDevice->Reset(&m_PresentParam)!=D3D_OK)
	{
		return	false;
	}

	m_pd3dDevice->GetDeviceCaps(&m_Caps);
	m_pD3DSprite->OnResetDevice();

	D3DXMatrixPerspectiveFovLH( &m_matProjection, D3DX_PI/4, GetScreenSizeAspect(), 1.0f, 10000.0f );
	InitMatrices();
	SetupRenderEnv();

	return	true;
}
POINT	CD3D90::GetBackBufferSize()
{
	POINT	pPoint;

	pPoint.x=m_PresentParam.BackBufferWidth;
	pPoint.y=m_PresentParam.BackBufferHeight;

	return	pPoint;
}


bool		CD3D90::CreateTexture(int iWidth,int iHeight,CTexture &Texture)
{

	DWORD	dwWidth=iWidth,dwHeight=iHeight;

	LPDIRECT3DTEXTURE9	pTexture;
	if(D3DXCreateTexture(m_pd3dDevice,dwWidth,dwHeight,0,0,D3DFMT_A8R8G8B8,D3DPOOL_MANAGED,&pTexture)!=D3D_OK)
	{
		return	false;
	}

	D3DSURFACE_DESC desc;
	pTexture->GetLevelDesc(0,&desc);

	Texture.SetTexture(pTexture);
	Texture.SetSize(desc.Width,desc.Height);
	Texture.SetReqSize(iWidth,iHeight);

	D3DXFilterTexture(Texture.GetTexture(),NULL,0,D3DX_FILTER_NONE);

	return	true;
}
//! NVASM  ϵ  vso  εϰ  ̴ Ѵ.
HRESULT	CD3D90::	CreateVSFromCompiledFile(D3DVERTEXELEMENT9 *pDecl,LPDIRECT3DVERTEXDECLARATION9 *pDeclaration,LPDIRECT3DVERTEXSHADER9 *pShader,TCHAR *strShaderFilename)
{
	HRESULT	hr;

    LPD3DXBUFFER pCode;
    if( FAILED( hr = m_pd3dDevice->CreateVertexDeclaration( pDecl, pDeclaration ) ) )
    {
        return hr;
    }
  
    DWORD dwFlags = 0;
#if defined( _DEBUG ) || defined( DEBUG )
    dwFlags |= D3DXSHADER_DEBUG;
#endif
    // Assemble the vertex shader from the file
    if( FAILED( hr = D3DXAssembleShaderFromFile( strShaderFilename, 
                                                 NULL, NULL, dwFlags, &pCode, NULL ) ) )
    {
  		MessageBox(NULL,DXGetErrorString9(hr),"D3DXAssembleShaderFromFile() Error",MB_OK);
		return hr;
    }

    // Create the vertex shader
    hr = m_pd3dDevice->CreateVertexShader( (DWORD*)pCode->GetBufferPointer(),
                                           pShader );
    pCode->Release();
    if( FAILED(hr) )
        return hr;

	return	S_OK;
}

//! NVASM  ϵ  vso  εϰ  ̴ Ѵ.
HRESULT	CD3D90::	CreateVSFromCompiledFile(LPDIRECT3DVERTEXSHADER9 *pShader,TCHAR *strShaderFilename)
{
	HRESULT	hr;

    LPD3DXBUFFER pCode;
  
    DWORD dwFlags = 0;
#if defined( _DEBUG ) || defined( DEBUG )
    dwFlags |= D3DXSHADER_DEBUG;
#endif
    // Assemble the vertex shader from the file
    if( FAILED( hr = D3DXAssembleShaderFromFile( strShaderFilename, 
                                                 NULL, NULL, dwFlags, &pCode, NULL ) ) )
    {
  		MessageBox(NULL,DXGetErrorString9(hr),"D3DXAssembleShaderFromFile() Error",MB_OK);
		return hr;
    }

    // Create the vertex shader
    hr = m_pd3dDevice->CreateVertexShader( (DWORD*)pCode->GetBufferPointer(),
                                           pShader );
    pCode->Release();
    if( FAILED(hr) )
        return hr;

	return	S_OK;
}
D3DXMATRIX CD3D90::GetFinalTransfromMatrix()
{
	D3DXMATRIX	matFinal,matTemp;
	D3DXMatrixMultiply(&matTemp,&m_matWorld,&m_matView);
	D3DXMatrixMultiply(&matFinal,&matTemp,&m_matProjection);
	D3DXMatrixTranspose( &matFinal, &matFinal );
	return	matFinal;
}
D3DXMATRIX CD3D90::GetFinalTransformUnTransposedMatrix()
{
	D3DXMATRIX	matFinal,matTemp;
	D3DXMatrixMultiply(&matTemp,&m_matWorld,&m_matView);
	D3DXMatrixMultiply(&matFinal,&matTemp,&m_matProjection);
	return	matFinal;
}
int	CD3D90::GetMaxBoneCount()
{
	if(m_bIsSoftwareVertexProcessing) return 0;

	return	(m_Caps.MaxVertexShaderConst-13)/4;
}

//!	 ۸ Ѵ.
HRESULT	CD3D90::CreateVertexBuffer(DWORD	dwLength,DWORD dwUsage,DWORD dwFVF,D3DPOOL Pool,IDirect3DVertexBuffer9** ppVertexBuffer,HANDLE *pSharedHandle)
{
	HRESULT hr;

	hr=m_pd3dDevice->CreateVertexBuffer(dwLength,dwUsage,dwFVF,Pool,ppVertexBuffer,pSharedHandle);
	if(FAILED(hr))
	{
		if(Pool==D3DPOOL_DEFAULT)
		{
			hr=m_pd3dDevice->CreateVertexBuffer(dwLength,dwUsage,dwFVF,D3DPOOL_MANAGED,ppVertexBuffer,pSharedHandle);
			if(FAILED(hr))
			{
				MessageBox(NULL,DXGetErrorString9(hr),"Create Vertex Buffer Failed",MB_OK);
				return	hr;
			}
			else
			{
				m_dwTotalVertexBufferMemorySize[MT_MANAGE]+=dwLength;
			}
		}
		else
		{
			MessageBox(NULL,DXGetErrorString9(hr),"Create Vertex Buffer Failed",MB_OK);
			return	hr;
		}
	}
	else
		m_dwTotalVertexBufferMemorySize[MT_VIDEMO]+=dwLength;

	return	hr;
}
//!	ؽ ۸ Ѵ.
bool	CD3D90::ReleaseVertexBuffer(IDirect3DVertexBuffer9* pVertexBuffer)
{
	if(!pVertexBuffer) return true;

	D3DVERTEXBUFFER_DESC	Desc;
	pVertexBuffer->GetDesc(&Desc);

	switch(Desc.Pool)
	{
	case	D3DPOOL_DEFAULT:
		assert(m_dwTotalVertexBufferMemorySize[MT_VIDEMO]>=Desc.Size);
		m_dwTotalVertexBufferMemorySize[MT_VIDEMO]-=Desc.Size;
		break;
	case	D3DPOOL_MANAGED:
		assert(m_dwTotalVertexBufferMemorySize[MT_MANAGE]>=Desc.Size);
		m_dwTotalVertexBufferMemorySize[MT_MANAGE]-=Desc.Size;
		break;
	case	D3DPOOL_SYSTEMMEM:
		assert(m_dwTotalVertexBufferMemorySize[MT_SYSTEM]>=Desc.Size);
		m_dwTotalVertexBufferMemorySize[MT_SYSTEM]-=Desc.Size;
		break;

	}
	pVertexBuffer->Release();
	return	true;
}
//!	ε ۸ Ѵ.
HRESULT	CD3D90::CreateIndexBuffer(DWORD	dwLength,DWORD dwUsage,D3DFORMAT Format,D3DPOOL Pool,IDirect3DIndexBuffer9** ppIndexBuffer,HANDLE *pSharedHandle)
{
	HRESULT hr;

	hr=m_pd3dDevice->CreateIndexBuffer(dwLength,dwUsage,Format,Pool,ppIndexBuffer,pSharedHandle);
	if(FAILED(hr))
	{
		if(Pool==D3DPOOL_DEFAULT)
		{
			hr=m_pd3dDevice->CreateIndexBuffer(dwLength,dwUsage,Format,D3DPOOL_MANAGED,ppIndexBuffer,pSharedHandle);
			if(FAILED(hr))
			{
				MessageBox(NULL,DXGetErrorString9(hr),"Create Index Buffer Failed",MB_OK);
				return	hr;
			}
			else
			{
				m_dwTotalIndexBufferMemorySize[MT_MANAGE]+=dwLength;
			}
		}
		else
		{
			MessageBox(NULL,DXGetErrorString9(hr),"Create Index Buffer Failed",MB_OK);
			return	hr;
		}
	}
	else
		m_dwTotalIndexBufferMemorySize[MT_VIDEMO]+=dwLength;

	return	hr;
}
//!	ε ۸ Ѵ.
bool	CD3D90::ReleaseIndexBuffer(IDirect3DIndexBuffer9* pIndexBuffer)
{
	if(!pIndexBuffer) return true;

	D3DINDEXBUFFER_DESC	Desc;
	pIndexBuffer->GetDesc(&Desc);

	switch(Desc.Pool)
	{
	case	D3DPOOL_DEFAULT:
		assert(m_dwTotalIndexBufferMemorySize[MT_VIDEMO]>=Desc.Size);
		m_dwTotalIndexBufferMemorySize[MT_VIDEMO]-=Desc.Size;
		break;
	case	D3DPOOL_MANAGED:
		assert(m_dwTotalIndexBufferMemorySize[MT_MANAGE]>=Desc.Size);
		m_dwTotalIndexBufferMemorySize[MT_MANAGE]-=Desc.Size;
		break;
	case	D3DPOOL_SYSTEMMEM:
		assert(m_dwTotalIndexBufferMemorySize[MT_SYSTEM]>=Desc.Size);
		m_dwTotalIndexBufferMemorySize[MT_SYSTEM]-=Desc.Size;
		break;

	}
	pIndexBuffer->Release();
	return	true;

}
//*	Ϸκ ؽó εѴ.  ޼ҵ带   ؽó Managed  ޸𸮷 ȴ.
HRESULT CD3D90::CreateTextureFromFile(LPDIRECT3DTEXTURE9 *ppTexture,char *strFilename)
{
	return CreateTextureFromFileEx(ppTexture,strFilename,1,0,D3DFMT_UNKNOWN,D3DPOOL_DEFAULT,D3DX_DEFAULT,D3DX_DEFAULT,0,NULL,NULL);
}
//*	Ϸκ ؽó εѴ.  ɼ    ִ.
HRESULT	CD3D90::CreateTextureFromFileEx(LPDIRECT3DTEXTURE9 *ppTexture,char *strFilename,DWORD dwMipLevels,DWORD dwUsage,D3DFORMAT Format,D3DPOOL Pool,DWORD dwFilter,DWORD dwMipFilter,D3DCOLOR ColorKey,D3DXIMAGE_INFO *pSrcInfo,PALETTEENTRY *pPalette)
{
	Pool=D3DPOOL_MANAGED;
	HRESULT hr;

	hr=D3DXCreateTextureFromFileEx(m_pd3dDevice,strFilename,D3DX_DEFAULT,D3DX_DEFAULT,dwMipLevels,dwUsage,Format,Pool,dwFilter,dwMipFilter,ColorKey,pSrcInfo,pPalette,ppTexture);
	if(FAILED(hr))
	{
		if(Pool==D3DPOOL_DEFAULT)
		{
			hr=D3DXCreateTextureFromFileEx(m_pd3dDevice,strFilename,D3DX_DEFAULT,D3DX_DEFAULT,dwMipLevels,dwUsage,Format,D3DPOOL_MANAGED,dwFilter,dwMipFilter,ColorKey,pSrcInfo,pPalette,ppTexture);
			if(FAILED(hr))
			{
				char strMsg[100];
				sprintf(strMsg,"Create Texture From File[%s] Failed",strFilename);
				MessageBox(NULL,DXGetErrorString9(hr),strMsg,MB_OK);
				return	hr;
			}
		}
		else
		{
			char strMsg[100];
			sprintf(strMsg,"Create Texture From File[%s] Failed",strFilename);
			MessageBox(NULL,DXGetErrorString9(hr),strMsg,MB_OK);
			return	hr;
		}
	}

	int iTotalMipLevel=(*ppTexture)->GetLevelCount();
	D3DSURFACE_DESC	Desc;
	MEMORY_TYPE	MemType;
	DWORD	dwMemory;
	for(int i=0;i<iTotalMipLevel;i++)
	{
		(*ppTexture)->GetLevelDesc(i,&Desc);
		GetSurfaceMemoryInfo(Desc,dwMemory,MemType);
		m_dwTotalTextureMemorySize[MemType]+=dwMemory;
	}

	return	hr;
}
//*	ؽó Ѵ.
bool	CD3D90::ReleaseTexture(LPDIRECT3DTEXTURE9 pTexture)
{
	if(!pTexture) return true;

	int iTotalMipLevel=pTexture->GetLevelCount();
	D3DSURFACE_DESC	Desc;
	MEMORY_TYPE	MemType;
	DWORD	dwMemory;
	for(int i=0;i<iTotalMipLevel;i++)
	{
		pTexture->GetLevelDesc(i,&Desc);
		GetSurfaceMemoryInfo(Desc,dwMemory,MemType);
		assert(m_dwTotalTextureMemorySize[MemType]>=dwMemory);
		m_dwTotalTextureMemorySize[MemType]-=dwMemory;
		pTexture->Release();
	}

	return	true;
}

//*	D3DSURFACE_DESC  , ش ǥ ޸ Ÿ԰ ޸  Ѵ.
bool	CD3D90::GetSurfaceMemoryInfo(const D3DSURFACE_DESC &SrcDesc,DWORD &dwTotalMemory,MEMORY_TYPE &MemType)
{

	MemType=MT_UNKNOWN;
	switch(SrcDesc.Pool)
	{
	case	D3DPOOL_DEFAULT:	MemType=MT_VIDEMO;	break;
	case	D3DPOOL_MANAGED:	MemType=MT_MANAGE;	break;
	case	D3DPOOL_SYSTEMMEM :	MemType=MT_SYSTEM;	break;
	}
	assert(MemType!=MT_UNKNOWN);

	int	iPixelSize=-1;

	switch(SrcDesc.Format)
	{
	case	D3DFMT_R8G8B8:	iPixelSize=16;	break;
	case	D3DFMT_A8R8G8B8:	iPixelSize=32;	break;
	case	D3DFMT_X8R8G8B8:	iPixelSize=32;	break;
	case	D3DFMT_R5G6B5:	iPixelSize=16;	break;
	case	D3DFMT_X1R5G5B5:	iPixelSize=16;	break;
	case	D3DFMT_A1R5G5B5:	iPixelSize=16;	break;
	case	D3DFMT_A4R4G4B4:	iPixelSize=16;	break;
	case	D3DFMT_R3G3B2:	iPixelSize=8;	break;
	case	D3DFMT_A8:	iPixelSize=8;	break;
	case	D3DFMT_A8R3G3B2:	iPixelSize=16;	break;
	case	D3DFMT_X4R4G4B4:	iPixelSize=16;	break;
	case	D3DFMT_A2B10G10R10:	iPixelSize=32;	break;
	case	D3DFMT_A8B8G8R8:	iPixelSize=32;	break;
	case	D3DFMT_X8B8G8R8:	iPixelSize=32;	break;
	case	D3DFMT_G16R16:	iPixelSize=32;	break;
	case	D3DFMT_A2R10G10B10:	iPixelSize=32;	break;
	case	D3DFMT_A16B16G16R16:	iPixelSize=64;	break;
	case	D3DFMT_A8P8:	iPixelSize=16;	break;
	case	D3DFMT_P8:	iPixelSize=8;	break;
	case	D3DFMT_L8:	iPixelSize=8;	break;
	case	D3DFMT_L16:	iPixelSize=16;	break;
	case	D3DFMT_A8L8:	iPixelSize=16;	break;
	case	D3DFMT_A4L4:	iPixelSize=8;	break;
	};

	iPixelSize=iPixelSize/8;

	assert(iPixelSize>=0);

	dwTotalMemory=iPixelSize*SrcDesc.Width*SrcDesc.Height;

	return	true;
}
