#include "GraphicObject.H"
#include "CommonUtility.H"
#include "ResourceManager.H"
#include "SFXML.H"

void	CGraphicObjectData::Create()
{
	m_fVersion=0.0f;
	m_iTotalAction=0;
	m_iTotalTexture=0;
	m_iTotalSound=0;
	m_pDescXML=NULL;
	m_ppaTextureResource=NULL;
	m_ppaSoundResource=NULL;
	m_pActions=NULL;
	m_dwRefCount=0;
	m_strXMLFileName[0]='\0';
}

void	CGraphicObjectData::Destroy()
{
	if(CResourceMgr::GetSingletonPtr())
	{
		if(m_pDescXML ) g_ResourceMgr.ReleaseResource(m_pDescXML);
		if(m_ppaTextureResource) 
		{
			for(int i=0;i<m_iTotalTexture;i++)
			{
				g_ResourceMgr.ReleaseResource(*(m_ppaTextureResource+i));
			}
		}

		if(m_ppaSoundResource)
		{
			for(int i=0;i<m_iTotalSound;i++)
			{
				g_ResourceMgr.ReleaseResource(*(m_ppaSoundResource+i));
			}
		}
	}

	if(m_ppaTextureResource) delete [] m_ppaTextureResource; m_ppaTextureResource=NULL;
	if(m_ppaSoundResource) delete [] m_ppaSoundResource;	m_ppaSoundResource=NULL;

	if(m_pActions) delete []m_pActions;
	m_pActions=NULL;

}

bool	CGraphicObjectData::LoadFromXML(char *strXMLFileName,bool bLoadImmediately)
{
	//	Critical Section ȿ  ޼ҵ带 ȣ ؼ ! ȵȴ.

	assert(strXMLFileName);
	assert(strlen(strXMLFileName)<=MAX_RES_FILENAME_LENGTH);

	if(bLoadImmediately)
	{
		m_pDescXML = (CXMLResource*)g_ResourceMgr.LoadResource(CResource::RT_XML,strXMLFileName,true);
		if(!SetDataFromXML())
		{
			g_ResourceMgr.ReleaseResource(m_pDescXML);
			m_pDescXML = NULL;

			MyMessageBox("SetDataFromXML() Failed");
			return false;
		}
	}
	return	true;
}
bool	CGraphicObjectData::SetDataFromXML()
{
	if(m_pDescXML == NULL || m_pDescXML->IsLoaded() == false) return false;

	assert(m_pDescXML->GetXMLContent());

	SFXMLContentData	*pRootContentData = m_pDescXML->GetXMLContent()->GetContentData();
	assert(pRootContentData);

	//	check this is really Graphic Object XML File
	if(strcmp(pRootContentData->GetDataName(),"GRAPHIC_OBJECT_DATA")!=0)
	{
		MyMessageBox("%s is not Graphic Object XML File",GetXMLFileName());
		return	false;
	}

	SFXMLContentAttributeNode	*pAttrNode;

	//	Read Version
	pAttrNode = pRootContentData->GetAttribute()->GetAttributeNodeByName("Version");
	if(pAttrNode)	m_fVersion = pAttrNode->GetValueByFloat();

	//	Read Total Action
	pAttrNode = pRootContentData->GetAttribute()->GetAttributeNodeByName("Total_Action");
	if(pAttrNode)	m_iTotalAction = pAttrNode->GetValueByInt();

	//	Read Total Texture
	pAttrNode = pRootContentData->GetAttribute()->GetAttributeNodeByName("Total_Texture");
	if(pAttrNode)	m_iTotalTexture = pAttrNode->GetValueByInt();

	//	Read Total Sound
	pAttrNode = pRootContentData->GetAttribute()->GetAttributeNodeByName("Total_Sound");
	if(pAttrNode)	m_iTotalSound = pAttrNode->GetValueByInt();

	if(m_iTotalAction <= 0)
	{
		MyMessageBox("%s has no action",GetXMLFileName());
		return false;
	}
	
	if(m_iTotalTexture>0)	
	{
		m_ppaTextureResource = new CTextureResource*[m_iTotalTexture];
		memset(m_ppaTextureResource,0,4*m_iTotalTexture);
	}
	if(m_iTotalSound>0) 
	{
		m_ppaSoundResource = new CSoundResource*[m_iTotalSound];
		memset(m_ppaSoundResource,0,4*m_iTotalSound);
	}
	
	m_pActions = new CActionData[m_iTotalAction];

	int	iTextureCount = 0;
	int	iSoundCount = 0;
	int	iActionCount = 0;
	int	iID;
	char	strFileName[MAX_RES_FILENAME_LENGTH+1];
	bool	bLoadImme;

	SFXMLContentData	*pContentData;
	for(int i=0;i<pRootContentData->GetTotalChildNumber();i++)
	{
		pContentData = pRootContentData->GetChildData(i);

		if(strcmp(pContentData->GetDataName(),"Texture")==0)
		{
			//	Read ID
			pAttrNode = pContentData->GetAttribute()->GetAttributeNodeByName("ID");
			assert(pAttrNode);

			iID = pAttrNode->GetValueByInt();

			assert(*(m_ppaTextureResource+iID) == NULL);

			//	Read File Path
			pAttrNode = pContentData->GetAttribute()->GetAttributeNodeByName("Path");
			assert(pAttrNode);

			strcpy(strFileName,pAttrNode->GetValue());
			
			//	Read Immediatley
			pAttrNode = pContentData->GetAttribute()->GetAttributeNodeByName("Immediatley");
			assert(pAttrNode);

			bLoadImme = strcmp(pAttrNode->GetName(),"true")==0;

			if(bLoadImme)	
				*(m_ppaTextureResource+iID)=(CTextureResource*)g_ResourceMgr.LoadResource(CResource::RT_TEXTURE,strFileName);
			else
				*(m_ppaTextureResource+iID)=(CTextureResource*)g_ResourceMgr.ReserveResource(CResource::RT_TEXTURE,strFileName);

			continue;

		}
		if(strcmp(pContentData->GetDataName(),"Sound")==0)
		{
			//	Read ID
			pAttrNode = pContentData->GetAttribute()->GetAttributeNodeByName("ID");
			assert(pAttrNode);

			iID = pAttrNode->GetValueByInt();

			assert(*(m_ppaTextureResource+iID) == NULL);

			//	Read File Path
			pAttrNode = pContentData->GetAttribute()->GetAttributeNodeByName("Path");
			assert(pAttrNode);

			strcpy(strFileName,pAttrNode->GetValue());
			
			//	Read Immediatley
			pAttrNode = pContentData->GetAttribute()->GetAttributeNodeByName("Immediatley");
			assert(pAttrNode);

			bLoadImme = strcmp(pAttrNode->GetName(),"true")==0;

			if(bLoadImme)	
				*(m_ppaSoundResource+iID)=(CSoundResource*)g_ResourceMgr.LoadResource(CResource::RT_SOUND,strFileName);
			else
				*(m_ppaSoundResource+iID)=(CSoundResource*)g_ResourceMgr.ReserveResource(CResource::RT_SOUND,strFileName);

			continue;
		}

		if(strcmp(pContentData->GetDataName(),"Action")==0)
		{
		}

	}

	return	true;
}


//////////////////////////////////////////////////////////////////////////////////////////////////
//	CGraphicObjectDataMgr
//////////////////////////////////////////////////////////////////////////////////////////////////
CGraphicObjectDataMgr* CGraphicObjectDataMgr::ms_Singleton;
CGraphicObjectData*	CGraphicObjectDataMgr::CreateNewGOData(char *strGODataXMLFile,bool bLoadImmediately)
{
	CGraphicObjectData	*pGOData = GetGOData(strGODataXMLFile);
	if(pGOData)
	{
		pGOData->AddRefCount();
		return pGOData;
	}

	pGOData = new CGraphicObjectData;
	m_vGOData.push_back(pGOData);

	if(!pGOData->LoadFromXML(strGODataXMLFile,bLoadImmediately))
	{
		MyMessageBox("GODataLoadFailed, XMLFile : %s",strGODataXMLFile);
		return NULL;
	}

	return	pGOData;
}
CGraphicObjectData*	CGraphicObjectDataMgr::GetGOData(char *strGODataXMLFile)
{
	CGraphicObjectData	*pGOData;
	for(GODataVectorItor Itor = m_vGOData.begin();Itor != m_vGOData.end(); Itor ++)
	{
		pGOData = (*Itor);
		if(strcmp(pGOData->GetXMLFileName(),strGODataXMLFile)==0) return pGOData;
	}

	return	NULL;
}
bool	CGraphicObjectDataMgr::ReleaseGOData(CGraphicObjectData *pGOData)
{
	for(GODataVectorItor Itor = m_vGOData.begin();Itor != m_vGOData.end(); Itor ++)
	{
		if( pGOData == (*Itor))
		{
			delete pGOData;
			m_vGOData.erase(Itor);
			return true;
		}
	}
	return	true;
}

void	CGraphicObjectDataMgr::Create()
{
}
void	CGraphicObjectDataMgr::Destroy()
{
	for(GODataVectorItor Itor = m_vGOData.begin();Itor != m_vGOData.end(); Itor ++)
	{
		delete (*Itor);
	}
}