
#include "Interface.H"
#include "ResourceManager.H"
#include "SFXML.H"
#include "CommonUtility.H"
#include "Common.H"
#include "Direct3D.H"

bool	CCard::GetInput()
{
	for(CardListItor Itor = m_ChildCard.begin();Itor != m_ChildCard.end(); Itor ++)
	{
		if((*Itor)->IsGetInputEnable()) (*Itor)->GetInput();
	}
	
	return	true;
}
bool	CCard::Process()
{
	BuildCardFromXML();

	for(CardListItor Itor = m_ChildCard.begin();Itor != m_ChildCard.end(); Itor ++)
	{
		if((*Itor)->IsProcessEnable()) (*Itor)->Process();
	}
	
	return	true;
}
bool	CCard::Render()
{
	if(m_pBGImage && m_pBGImage->IsLoaded())
	{
		D3DSURFACE_DESC Desc;
		m_pBGImage->GetTexture()->GetLevelDesc(0,&Desc);

		g_D3D90.GetSprite()->Begin();

		g_D3D90.GetSprite()->Draw(m_pBGImage->GetTexture(),NULL,NULL,NULL,0,NULL,0xffffffff);

		g_D3D90.GetSprite()->End();
	}

	for(CardListItor Itor = m_ChildCard.begin();Itor != m_ChildCard.end(); Itor ++)
	{
		if((*Itor)->IsRenderEnable())	(*Itor)->Render();
	}

	return	true;
}
bool	CCard::Create()
{
	m_ptPosition.x=0;
	m_ptPosition.y=0;

	m_ptSize.x=0;
	m_ptSize.y=0;

	m_bRender = true;
	m_bProcess = true;
	m_bGetInput = true;

	m_pCardDescXML = NULL;
	m_pBGImage = NULL;
	m_pBGM = NULL;
	return	true;
}
void	CCard::BuildCardFromXML()
{
	if(!m_pCardDescXML || m_pCardDescXML->IsLoaded() == false) return;

	SFXMLContent	*pContent = m_pCardDescXML->GetXMLContent();
	if(!pContent) return;

	SFXMLContentData	*pBaseContentData = pContent->GetContentData();
	if(!pBaseContentData)
	{
		MyMessageBox("CCard::BuildCardFromXML() Error! No Content Data in xmlfile %s\n",m_pCardDescXML->GetFileName());
		return;
	}

	if(strcmp(pBaseContentData->GetDataName(),"CARD_DATA")!=0)
	{
		MyMessageBox("CCard::BuildCardFromXML() Error!  It's not Card Description xmlfile %s\n",m_pCardDescXML->GetFileName());
		return ;
	}

	SFXMLContentData *pContentData;
	bool	bLoadImme;
	char	strFileName[MAX_RES_FILENAME_LENGTH+1];

	for(int i=0;i<pBaseContentData->GetTotalChildNumber();i++)
	{
		pContentData = pBaseContentData->GetChildData(i);
		if(strcmp(pContentData->GetDataName(),"BG_TEXTURE")==0)
		{
			strcpy(strFileName,pContentData->GetAttrValue("Path"));
			bLoadImme = strcmp(pContentData->GetAttrValue("Immediately"),"true")==0;
			if(bLoadImme)
				m_pBGImage = (CTextureResource*)g_ResourceMgr.LoadResource(CResource::RT_TEXTURE,strFileName,true);
			else
				m_pBGImage = (CTextureResource*)g_ResourceMgr.ReserveResource(CResource::RT_TEXTURE,strFileName);
		}
		if(strcmp(pContentData->GetDataName(),"BG_MUSIC")==0)
		{
			strcpy(strFileName,pContentData->GetAttrValue("Path"));
			bLoadImme = strcmp(pContentData->GetAttrValue("Immediately"),"true")==0;
			if(bLoadImme)
				m_pBGM = (CMusicResource*)g_ResourceMgr.LoadResource(CResource::RT_MUSIC,strFileName,true);
			else
				m_pBGM = (CMusicResource*)g_ResourceMgr.ReserveResource(CResource::RT_MUSIC,strFileName);
		}

	}

	g_ResourceMgr.ReleaseResource(m_pCardDescXML);
	m_pCardDescXML = NULL;
}
void	CCard::SetCardDescXML(char *strXMLFileName)
{
	if(m_pCardDescXML)	g_ResourceMgr.ReleaseResource(m_pCardDescXML);
	m_pCardDescXML = (CXMLResource*)g_ResourceMgr.ReserveResource(CResource::RT_XML,strXMLFileName);
}
bool	CCard::PushCardBack(CCard *pChild)
{
	m_ChildCard.push_back(pChild);
	return	true;
}
bool	CCard::PushCardFront(CCard *pChild)
{
	m_ChildCard.push_front(pChild);
	return	true;
}
bool	CCard::ReleaseCard(CCard *pChild)
{
	for(CardListItor Itor = m_ChildCard.begin();Itor != m_ChildCard.end(); Itor ++)
	{
		if( (*Itor) == pChild)
		{
			delete *Itor;
			m_ChildCard.erase(Itor);
			return true;
		}
	}
	return	true;
}

int	CCard::GetTotalChildNumber()
{
	return	m_ChildCard.size();
}

bool	CCard::Destroy()
{
	if(CResourceMgr::GetSingletonPtr())
	{
		if(m_pCardDescXML)	g_ResourceMgr.ReleaseResource(m_pCardDescXML);
		if(m_pBGM)	g_ResourceMgr.ReleaseResource(m_pBGM);
		if(m_pBGImage) g_ResourceMgr.ReleaseResource(m_pBGImage);
	}

	m_pCardDescXML = NULL;
	m_pBGM = NULL;
	m_pBGImage = NULL;

	ReleaseAllChild();
	return	true;
}
bool	CCard::ReleaseAllChild()
{
	for(CardListItor Itor = m_ChildCard.begin();Itor != m_ChildCard.end(); Itor ++)
	{
		delete (*Itor);
	}
	m_ChildCard.clear();

	return	true;
}

bool	CCard::KeyDown(int iKeyID)
{
	CCard *pChild;
	for(CardListItor Itor = m_ChildCard.begin();Itor != m_ChildCard.end(); Itor ++)
	{
		pChild = *Itor;
		if(pChild->IsGetInputEnable()) 
		{
			if(pChild->KeyDown(iKeyID)) return true;
		}
	}
	
	return	false;
}
bool	CCard::MouseDown(BYTE byButton,int iMouseX,int iMouseY)
{
	for(CardListItor Itor = m_ChildCard.begin();Itor != m_ChildCard.end(); Itor ++)
	{
		if((*Itor)->IsGetInputEnable()) 
		{
			if((*Itor)->MouseDown(byButton,iMouseX,iMouseY)) return true;
		}
	}
	return	false;
}
bool	CCard::MouseUp(BYTE byButton,int iMouseX,int iMouseY)
{
	for(CardListItor Itor = m_ChildCard.begin();Itor != m_ChildCard.end(); Itor ++)
	{
		if((*Itor)->IsGetInputEnable()) 
		{
			if((*Itor)->MouseUp(byButton,iMouseX,iMouseY)) return true;
		}
	}
	return	false;
}
bool	CCard::MouseDClick(BYTE byButton,int iMouseX,int iMouseY)
{
	for(CardListItor Itor = m_ChildCard.begin();Itor != m_ChildCard.end(); Itor ++)
	{
		if((*Itor)->IsGetInputEnable()) 
		{
			if((*Itor)->MouseDClick(byButton,iMouseX,iMouseY)) return true;
		}
	}
	return	false;
}
bool	CCard::MouseWheel(int iAmount,int iMouseX,int iMouseY)
{
	for(CardListItor Itor = m_ChildCard.begin();Itor != m_ChildCard.end(); Itor ++)
	{
		if((*Itor)->IsGetInputEnable()) 
		{
			if((*Itor)->MouseWheel(iAmount,iMouseX,iMouseY)) return true;
		}
	}
	return	false;
}
